/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Eliot Setzer
 *   C. Greg Hagerty <cgreg@cgreg.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const ANNOZILLA_XUL_NS = "http://annozilla.mozdev.org/xul/2005";

// Each window has a single gAnnozilla instance.
// Annotation data itself is held in a data model associated with
// each tab; see model.js.
var gAnnozilla; // was set by initialize.js, now by initAnnozilla via overlay
var gAnnozillaBrowser; // for tab handling

function initAnnozilla () {
    gAnnozilla = new Annozilla();
    window.addEventListener("unload", 
                            function (e) {gAnnozilla.unload();},
                            false);

    gAnnozillaBrowser = window.top.gBrowser;
    var container = gAnnozillaBrowser.tabContainer;
    container.addEventListener("TabSelect", gAnnozilla.tabSelected, false);
    gBrowser.addEventListener("DOMContentLoaded", gAnnozilla.contentLoaded, true);
    
    /*
    // Test code:
    
    var rdf =
          Components.classes["@mozilla.org/rdf/rdf-service;1"].
          getService(Components.interfaces.nsIRDFService);
    var 
    datasource = rdf.GetDataSource
       ("chrome://annozilla/content/resources/annotationTypes.rdf");
        
    // TODO this is an async load
    
    var e = datasource.GetAllResources();
    Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService).
      logStringMessage ("annozilla.js e = " +e);
    while (e.hasMoreElements())
{
        var r = e.getNext();
        Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService).
      logStringMessage ("annozilla.js r = " +
        r.QueryInterface(Components.interfaces.nsIRDFResource).Value);
    }
    
    var uri = "http://www.w3.org/2000/10/annotationType#Comment";

    var source = datasource.GetSource (
            rdf.GetResource ("http://home.netscape.com/NC-rdf#URL"),
            rdf.GetLiteral (uri),
            true);
    
         Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService).
      logStringMessage ("annozilla.js SOURCE " + source.QueryInterface (Components.interfaces.nsIRDFLiteral).Value);


      var target = datasource.GetTarget(
            rdf.GetResource (uri),
            rdf.GetResource ("http://home.netscape.com/NC-rdf#Name"),
            true);
            
      Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService).
      logStringMessage ("annozilla.js TARGET " + target.QueryInterface (Components.interfaces.nsIRDFLiteral).Value);
    */
}

function runAnnozillaWizard () {
/*
    var prefBranch = 
        Components.classes["@mozilla.org/preferences;1"].
           getService(Components.interfaces.nsIPrefService).
           getBranch("annozilla.");

    if ( ! prefBranch.prefHasUserValue("wizard.run") ||
         ! prefBranch.getBoolPref ("wizard.run"))
    {
        window.openDialog ('chrome://annozilla/content/prefs/wizard/prefwizard.xul');
    }
*/
}

function Annozilla() {
    
    if (! Annozilla._hasRequiredComponents()) {
        return;
    }

    this.windowIndex = 0;

    this.bundle = srGetStrBundle("chrome://annozilla/locale/annozilla.properties");

    // initialize annotations service
    this.annotService =
       Components.classes[Annozilla.ANNOTATIONS_SERVICE_CONTRACTID].getService().
           QueryInterface(Components.interfaces.nsIAnnotationsService);

    // get the annozilla AnnotationsApplication object
    this.app = this.annotService.registerApplication("annozilla");

    // add our observer
    this.app.addObserver(this);

    // icons module
    this.icons = new AnnozillaIconManager(this);

    // sidebar module, when loaded
    this.sidebar = null;

    // prefs module
    this.prefs = new AnnozillaPrefs(this);

    // This is necessary to prevent _addAnnotationToContainer from failing when
    // an annotation is created before any annotations have ever been loaded by
    // the sidebar. TODO: only deals with the current tab
    // Action: delete this and ensure that the model handles this scenario.
    //this.clearAnnotations(gBrowser.selectedBrowser);

    this.defaultBodyDestination = window.document.webNavigation;

    // Semaphore allows user to stop load by setting.
    // TODO should be per-tab?
    this.asynchLoading = false;

}

Annozilla.UNKNOWN_REASON_URL = "about:blank";

Annozilla.ANNOTATIONS_SERVICE_CONTRACTID = "@mozilla.org/annotations_service;1";
Annozilla.XPOINTER_SERVICE_CONTRACTID = "@mozilla.org/xpointer-service;1";

Annozilla.ANNOT_NS = "http://www.w3.org/2000/10/annotation-ns#";
Annozilla.RDF_NS   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
Annozilla.DC10_NS    = "http://purl.org/dc/elements/1.0/";
Annozilla.DC11_NS    = "http://purl.org/dc/elements/1.1/";
Annozilla.THREAD_NS= "http://www.w3.org/2001/03/thread#";
Annozilla.XHTML_NS = "http://www.w3.org/1999/xhtml";
Annozilla.TYPE_NS = "http://www.w3.org/2000/10/annotationType#";

// The previous value for this ("annotation-body") produced warnings (about the
// '-' not being a legal character) in a version of Firefox compiled with
// debugging messages.
Annozilla.WINDOW_ID_PREFIX = "annotationBody";

Annozilla.prototype = {
    windowIndex: undefined,
    bundle: undefined,
    defaultBodyDestination: undefined
};

Annozilla.prototype.logToConsole = function (message)
{
    Components.classes['@mozilla.org/consoleservice;1'].
        getService(Components.interfaces.nsIConsoleService).
          logStringMessage (message);
};

Annozilla.prototype.clearAnnotations = function (target)
{
    new AnnozillaAnnotationModel (target).clearAnnotations();
};

Annozilla.prototype.unload = function() {
    // remove observer
    this.app.removeObserver(this);

    // tell prefs we're shutting down
    this.prefs.destructor();
    
    var container = gAnnozillaBrowser.tabContainer;
    container.removeEventListener("TabSelect", gAnnozilla.tabSelected, false);
    
    gBrowser.removeEventListener("DOMContentLoaded", gAnnozilla.contentLoaded, true);

};

Annozilla.prototype.QueryInterface = function(iid) {
    if (! (iid.equals(Components.interfaces.nsIAnnotationsObserver) ||
           iid.equals(Components.interfaces.nsISupports) ) ) {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
};

Annozilla.prototype.insertIconsPrefChanging = function(insert){
    if(this.icons.insertIconsPrefChanging){
        this.icons.insertIconsPrefChanging(insert);
    }
};

// Return the range specified by the fragment.
Annozilla.prototype.resolveFragment =
function(fragment, target) {
    return gAnnozillaXPointerUtils.xptrResolveFragment(fragment, target.contentDocument);
};

// Return the range to which the annotation applies.
Annozilla.prototype.resolveAnnotation =
function(annotation, target) {
    return gAnnozillaXPointerUtils.xptrResolveAnnotation(annotation, target.contentDocument);
};

/**
 * Determine whether the Mozilla Composer HTML editor is available.
 */
Annozilla.prototype.isMozillaEditorAvailable = function () {
/* Prefered test is to check whether the htmleditor class is installed;
   however even on MozillaFirebird 1.0 this test returns true.
   Backup test is to test for the existence of the element "key_editPage"
   in the top browser window - this is associated with the "File ->
   Edit Page" menu option. Hopefully this hackiness will be removed
   at some point in the future.
 */
    return (("@mozilla.org/editor/htmleditor;1" in Components.classes) &&
            window.top.document.getElementById ("key_editPage") != undefined);
};

/**
 * Notification that a document has finished loading and constructed its DOM.
 * If autoloading is enabled, then start loading annotations for the page.
 */
Annozilla.prototype.contentLoaded = function (event)
{
    // TODO This seems to fail when pages are restored at startup.    
    var url = event.currentTarget.mCurrentBrowser.contentDocument.location.href;
    
    // TODO frame detection? non-HTML doc types?
    if (gAnnozilla.autoLoadAllowSite(url)) {
        gAnnozilla.userLoadOnCommand (event.currentTarget.mCurrentBrowser);
    }
};

// Determine whether an annotation is associated with a Local Server
// (Should be a method on the annotation?)
Annozilla.prototype._isLocal = function (annotation)
{
    return annotation.serverName == this.app.LOCALSERVER_URI;
};

// Generate a URI for the body of an annotation.
// For a local annotation, create a data: URI for the body.
// For a remote annotation, just use the existing URI for the body.
Annozilla.prototype._generateURIForAnnotation = function (annotation)
{
    if (this._isLocal (annotation)) {
        return LocalServerUtils.createTempURIFromString(annotation.getBodyText());
    } else {
        return annotation.bodyURI;
    }
};

/*
 * Currently, annotation collection is cleared and reloaded for each tab switch
 * But _addAnnotations/_addNextAnnotation checks each annotation URI is for current page
 *  so that if we want to avoid reload we could accumulate annotations
 *  for open pages by not resetting the collection
 */
Annozilla.prototype.tabSelected = function (event) {

    var browser = gAnnozillaBrowser.selectedBrowser;
    // browser is the newly selected tab

    var url = content.document.location.toString();

// TODO why not "this.sidebar"? (because "this" refers to an XUL element when called)
    if (gAnnozilla.sidebar)
    {
        gAnnozilla.sidebar.tabSelected (browser);
    }

};

/**
 * Return the chrome URL used for editing annotation bodies.
 */
Annozilla.prototype.getEditorUrl = function () {
    return this.isMozillaEditorAvailable() ?
           "chrome://annozilla/content/editor/composer.xul" :
           "chrome://annozilla/content/editor/editor.xul";
};

Annozilla.prototype.removeIcons = function(target) {
    this.icons.removeIcons(target);
};

/**
 * Loads annotations for the current page.
 *
 * @param target Optional argument indicating the browser
 * in which the load occurs.
 */   
Annozilla.prototype.userLoadOnCommand = function(target) {
    
    // If no target was specified, get the selected tab.    
    if (target == undefined)
    {
        target = gBrowser.selectedBrowser;
    }
    
    //this.userLoading = true;

    this.icons.removeIcons(target);

    // Finish if nothing to do
    if (this.currentServerCount() === 0) {
        return;
    }

    this.clearAnnotations(target);
    
    var model = new AnnozillaAnnotationModel (target);
    var serverCount = model.startOperation (this.currentServerCount());
    if (this.sidebar)
    {
        this.sidebar.updateProgress (serverCount, target);
    }

    // Do the load. We will be notified of the results by callbacks.
    this.app.loadAnnotations (
        this.stripURL(target.contentDocument.location.href),
        gBrowser.selectedBrowser);
        
    /* Experimental frameset support:    
    var frames = target.contentWindow.frames;
    for (var i = 0; i < frames.length; i++) { 
    
        Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService)
          .logStringMessage("location " + i + "="+frames[i].document.location.href);
          
        model.startOperation (this.currentServerCount());
        
        this.app.loadAnnotations
            (this.stripURL(frames[i].document.location.href),
             gBrowser.selectedBrowser);
    }
    */
};

Annozilla.prototype.checkForRepliesOnCommand = function(uri, target) {

    var model = new AnnozillaAnnotationModel (target);

    if (!model.hasAnnotation(uri))
    {
        window.alert (this.bundle.GetStringFromName ("INTERNAL_ERROR"));
        return;
    }

    var serverCount = model.startOperation (this.currentServerCount());
    if (this.sidebar)
    {
        this.sidebar.updateProgress (serverCount, target);
    }

    var annotation = model.getAnnotation(uri).annot;

    this.app.getReplies (annotation, gBrowser.selectedBrowser);
};

Annozilla.prototype.openEditorWindow = function(
    windowOptions,
    annotation,
    baseURL,
    fragment,
    bodyURI,
    inReplyTo,
    inReplyToBodyURI,
    threadRoot,
    target) {
    
    var w = window.openDialog(
        this.getEditorUrl(),
        Annozilla.WINDOW_ID_PREFIX + this.windowIndex++,
        windowOptions,
        annotation,
        baseURL,
        fragment,
        bodyURI,
        inReplyTo,
        inReplyToBodyURI,
        threadRoot,
        target);
    
    if(fragment) {
        // Fails because the icon manager requires an annotation
        // and we don't have one
        this.icons.addIconForEditorWindow(w, fragment, target);
    } else if (annotation == undefined) {
        throw("openEditorWindow requires either a fragment or an existing annotation");
    }
};

Annozilla.prototype.post = function(baseURL, fragment, target){
    this.openEditorWindow(
        "chrome,resizable,dialog=no",
        null, // annotation
        baseURL,
        fragment,
        null, // bodyURI,
        null, // inReplyTo
        null, // inReplyToBodyURI
        null, // threadRoot
        target);
};

Annozilla.prototype.postOnCommand = function() {

    // get the xptr
    // TODO assumes the current tab, is this OK?
    var seln = window._content.getSelection();
    /*
    if (seln.anchorNode == null) {
        // it might be a good idea to move into html[1]/body[1]? if it exists.
    }
    */
 
    var baseURL = this.stripURL(window._content.location.href);
    var xptr = gAnnozillaXPointerUtils.xptrCreateXPointer(seln, window._content.document);
    this.post(baseURL, xptr, gBrowser.selectedBrowser);
};

Annozilla.prototype.replyOnCommand = function(replyToURI) {
    alert ("TODO");
};

Annozilla.prototype.update = function(annotationURI, target) {
    
    var model = new AnnozillaAnnotationModel (target);
    
    if (! model.hasAnnotation(annotationURI) ) {
        window.alert(this.bundle.GetStringFromName ("EDIT_INTERNAL_ERROR"));
        return;
    }

    var annotation = model.getAnnotation(annotationURI);
    var bodyURI = this._generateURIForAnnotation(annotation.annot);

    // If the annotation has a thread root set, use it.
    // Otherwise, we are editing an annotation, not a reply,
    // so use this annotation's URI as the root if we create any replies.
    this.openEditorWindow(
        "chrome,resizable,dialog=no",
        annotation.annot.clone(),
        null,
        null,
        bodyURI,
        annotation.inReplyTo,
        null, // inReplyToBodyURI
        annotation.threadRoot,
        target);
};

Annozilla.prototype.updateOnCommand = function(annotationURI, target) {
    this.update(annotationURI, target);
};

Annozilla.prototype.deleteOnCommand = function(annotationURI, target) {

    var model = new AnnozillaAnnotationModel (target);
   
    if (! model.hasAnnotation(annotationURI)) {
        window.alert(this.bundle.GetStringFromName ("DELETE_INTERNAL_ERROR"));
        return;
    }

    var serverCount = model.startOperation (1); // always just 1 server for delete
    if (this.sidebar)
    {
        this.sidebar.updateProgress (serverCount, target);
    }

    var annotation = model.getAnnotation(annotationURI).annot;
    this.app.deleteAnnotation(annotation, target);

};

Annozilla.prototype.loadBodyOnCommand = function(annotationURI, target) {
    
    var model = new AnnozillaAnnotationModel (target);
    
    if (! model.hasAnnotation(annotationURI)) {
        window.alert(this.bundle.GetStringFromName ("LOAD_INTERNAL_ERROR"));
        return;
    }
  
    var annotation = model.getAnnotation(annotationURI).annot;
    
    var bodyURI = this._generateURIForAnnotation(annotation);
  
    if (this.prefs.getPref("loadInNewWindow")) {
        window.open(bodyURI, "_blank","width=600,height=320,scrollbars=yes,resizable=yes","");
    }
    else {
        this.loadAnnotationBody(bodyURI);
    }
};

Annozilla.prototype.scrollIntoViewOnCommand = function(target) {
    
    var model = new AnnozillaAnnotationModel (target);
    var annotationURI = this.sidebar.getSelectedAnnotationURI();
  
    if ( ! model.hasAnnotation(annotationURI)) {
        window.alert(this.bundle.GetStringFromName ("LOAD_INTERNAL_ERROR"));
        return;
    }
    
    var annotation = model.getSelectedAnnotationURI(annotationURI).annot;
    var range = gAnnozillaXPointerUtils.xptrResolveAnnotation(annotation, target.contentDocument);
    if (range) {
        var startNode = range.startContainer;
        while (startNode.nodeType!=Node.ELEMENT_NODE) {
            startNode = startNode.parentNode;
        }
        startNode.scrollIntoView();
    }
};

Annozilla.prototype.autoLoadAllowSite = function(url){
    if (this.prefs.getPref("autoLoadMulti") == "never") {
        return false;
    }
    if (this.prefs.getPref("autoLoadMulti") == "sites") {
        var allowSite = false;
        var sites = this.prefs.getPref("autoLoadSites").split(" ");
        for (var ix=0; ix<sites.length; ix++) {
            if (url.indexOf(sites[ix]) === 0) {
                allowSite = true;
                dump("Autoload allowed for "+url+"\n");
                break;
            }
        }
        if (!allowSite) {
            return false;
        }
    }
    return true;
};

Annozilla.prototype.byeByeSidebar = function () {
    dump("Annozilla: Bye bye Sidebar\n");
    this.sidebar = null;
};

/**
 * Success callback function to return the results for a loadAnnotations request.
 * This function returns the annotations for one server, designated by serverURI.  
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri that the annotations annotate
 * @param annotations nsISimpleEnumerator of nsIAnnotations for annotatedURI
 *                    that were on serverURI annotations server
 *  @param target Target of the load
 */
Annozilla.prototype.loadAnnotationsOnSuccess =
function (serverURI, annotatedURI, annotationsLength, annotations, target) {

    try {

        var model = new AnnozillaAnnotationModel (target);
        var serverCount = model.serverResponded();
        
        // If we've moved on to a new page, ignore these annotations.
        // Work in progress: it's OK if we've moved on to another tab,
        // just load the annotations in the correct tab
        if (annotatedURI == target.currentURI.spec) {
            this._addAnnotations (annotations, target);
        } else {
            dump(
                 "ignoring loadAnnotationsOnSuccess of " + annotatedURI + " from " + 
                 serverURI + " because it pertains to old URI.  currentURI= " +
                 target.currentURI.spec + "\n");
        }

        if (this.sidebar)
        {
            this.sidebar.updateProgress (serverCount, target);
        }

    } catch (e) {
        var consoleService=Components.classes['@mozilla.org/consoleservice;1'];
        var console = consoleService.getService(Components.interfaces.nsIConsoleService);
        console.logStringMessage("Annozilla: error loading annotations");
        Components.utils.reportError (e);
    }

};

/**
 * Callback function to indicate failure for a loadAnnotations request.
 * This function indicates failure for one particular annotations server,
 * designated by serverURI.
 *
 * @param serverURI string uri of the annotations server responding
 * @param annotatedURI string uri for which annotations were requested
 * @param message string error message
 */
Annozilla.prototype.loadAnnotationsOnFailure = function(serverURI, annotatedURI, message) {
    dump("Annozilla: loadAnnotationsOnFailure for server: " + serverURI + " w/ no annotations of " + annotatedURI +
         " because of error: " + message + "\n");
};

/**
 * Called when an annotation post request succeeds.
 *
 * @param newAnnotation nsIAnnotation the newly created annotation
 */
Annozilla.prototype.postAnnotationOnSuccess = function(newAnnotation, target) {

    // if this annotation applies to a page we're no longer viewing
    var uri = newAnnotation.data.getProperty(Annozilla.ANNOT_NS, "annotates");
    if (this.stripURL(uri) != this.stripURL(target.contentDocument.location.href)) {
        dump("posted annotation applies to page no longer being viewed\n");
        dump(" ('"+uri+"' while viewing '"+target.contentDocument.location.href+"')\n");
        return;
    }

    var range = this.resolveAnnotation(newAnnotation, target);

    new AnnozillaAnnotationModel (target).addAnnotation (newAnnotation, range);

    // calling AnnozillaIconManager here avoids additional resolve
    this.icons.addAnnotation(newAnnotation, range, target.contentDocument, target);

    // ditto sidebar
    if (this.sidebar) {
        // TODO check on correct tab?
        this.sidebar.addAnnotation(newAnnotation, range);
    }
};

/**
 * Called when an annotation post fails.
 *
 * @param message string error message
 */
Annozilla.prototype.postAnnotationOnFailure = function(message) {};

/**
 * Called when an annotation update succeeds.
 *
 * @param updatedAnnotation nsIAnnotation that has been updated on server
 */
Annozilla.prototype.updateAnnotationOnSuccess = function(updatedAnnotation, target) {
    
    var model = new AnnozillaAnnotationModel (target);
    model.updateAnnotation (updatedAnnotation);

};

/**
 * Called when an annotation update fails.
 *
 * @param url string url of the annotation that could not be updated
 * @param message string error message
 */
Annozilla.prototype.updateAnnotationOnFailure =
function(url, message, target) {};

/**
 * Called when an annotation is successfully deleted.
 *
 * @param url string url of deleted annotation
 */
Annozilla.prototype.deleteAnnotationOnSuccess = function(url, target) {
    
    var model = new AnnozillaAnnotationModel (target);
    var serverCount = model.serverResponded();
    
    model.deleteAnnotation(url);
    
    if (this.sidebar)
    {
        this.sidebar.updateProgress (serverCount, target);
    }

};

/**
 * Called when an annotation delete fails.
 *
 * @param url string url of the annotation that could not be deleted
 * @param message string error message
 */
Annozilla.prototype.deleteAnnotationOnFailure = function(url, message, target) {
    
    var model = new AnnozillaAnnotationModel (target);
    var serverCount = model.serverResponded();
    if (this.sidebar)
    {
        this.sidebar.updateProgress (serverCount, target);
    }
    window.alert (this.bundle.formatStringFromName ("DELETE_FAIL_ERROR", [url, message], 2));
};

/**
 * Called when the replies to an annotation have been successfully
 * loaded.
 * 
 * @param annotation nsIAnnotation for which replies were requested
 * @param repliesCount Number of replies
 * @param replies Array of nsIAnnotation replies
 * @param target Target with which the replies are associated
 */
Annozilla.prototype.getRepliesOnSuccess =
function(annotation, repliesCount, replies, target) {
    
    // TODO If we've moved on to a new page, ignore these replies
    try {

        var model = new AnnozillaAnnotationModel (target);
        var serverCount = model.serverResponded();

        this._addReplies (annotation, repliesCount, replies, target);

        if (this.sidebar)
        {
            this.sidebar.updateProgress (serverCount, target);
        }

    } catch (e) {
        var consoleService=Components.classes['@mozilla.org/consoleservice;1'];
        var console = consoleService.getService(Components.interfaces.nsIConsoleService);
        console.logStringMessage("Annozilla: error loading annotations");
        Components.utils.reportError (e);
    }
};

/**
 * Called when getting the replies to an annotation fails.
 *
 * @param annotation nsIAnnotation for which replies were requested
 * @param message string error message
 */
Annozilla.prototype.getRepliesOnFailure =
function(annotation, message, target) {
    Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService).
      logStringMessage ("Annozilla.getRepliesOnFailure");
};

/**
 * Called an annotation has successfully been replied to.
 * 
 * @param original nsIAnnotation that existed originally
 * @param reply nsIAnnotation reply created
 */
Annozilla.prototype.replyToAnnotationOnSuccess =
function(original, reply, target) {
    this._addReplies (original, 1, [ reply ], target);
};

/**
 * Called when replying to an annotation fails.
 *
 * @param original nsIAnnotation to which we attempted to reply
 * @param message string error message
 */
Annozilla.prototype.replyToAnnotationOnFailure =
function(original, message, target) {};

/**
 * Called after a loadAnnotationBody request
 * when the body text of the annotation has been successfully loaded.
 *
 * @param annotation nsIAnnotation with updated body text
 * @param bodyText string text of the annotation's body
 */
Annozilla.prototype.loadAnnotationBodyOnSuccess =
function(annotation, bodyText) {};

/**
 * Called after a loadAnnotationBody request when
 * the body text fails to load.
 *
 * @param annotation nsIAnnotation that body was requested for (unmodified)
 * @param message string error message
 */
Annozilla.prototype.loadAnnotationBodyOnFailure =
function(annotation, message) {};

Annozilla.prototype.setDefaultBodyDestination = function(destination){
    this.defaultBodyDestination = destination;
};

Annozilla.prototype.loadAnnotationBody = function(bodyURI, destination){
    if(destination) destination.loadURI(bodyURI);
    else this.defaultBodyDestination.loadURI(bodyURI);
};

Annozilla._hasRequiredComponents = function() {
    if (! Components.classes[Annozilla.ANNOTATIONS_SERVICE_CONTRACTID]) {
        return false;
    }
    if (! Components.classes[Annozilla.XPOINTER_SERVICE_CONTRACTID]) {
        return false;
    }
    return true;
};

/**
 * Private function to strip query strings off of URLs. (we don't want to be
 * nosy!)
 * Also strips off fragment IDs.
 *
 * @param url String URL to strip query strings from
 * @return URL string stripped of queries
 */
Annozilla.prototype.stripURL = function (url) {
    if(!url || ('string'!= typeof(url)) || ('' == url) || url == 'about:blank'){
        return 'about:blank';
    }

    // Strip off any query strings (Don't want to be too nosy).
    var queryPart = url.indexOf('?');
    if (queryPart != -1) {
	url = url.slice(0, queryPart);
    }

    url = url.replace (/#.*/,'');

    return url;    
};

Annozilla.prototype.isAutoLoadEnabled = function() {
    return (this.prefs.getPref("autoLoadMulti") != "never");
};

Annozilla.prototype.currentServerCount = function(){
    return this.app.getServerCount();
};

Annozilla.prototype._addAnnotationToContainer = function (aAnnotation, aRange, aContainer) {
    aContainer[aAnnotation.annotationURI] = {
        annot: aAnnotation,
        range: aRange
    };
};

Annozilla.prototype._debugBadAnnotation = function(annotation) {
    dump("annotation.data is NULL.  enumerating properties:\n");
    dump("serverName: " + annotation.serverName + "\n");
    dump("annotationURI: " + annotation.annotationURI + "\n");
    dump("bodyURI: " + annotation.bodyURI + "\n");
    dump("getBodyText: " + annotation.getBodyText() + "\n");
    dump("setting bodyText.\n");
    annotation.setBodyText("New test body text");
    dump("new body text: " + annotation.getBodyText() + "\n");
};

Annozilla.prototype.userStopLoadCommand = function () {
    this.asynchLoading = false; // TODO make per-tab
    //TODO: SIDEBAR
    //var menuitem = window.document.getElementById('stop-load');
    //menuitem.hidden = 1;
    //this.operationComplete("LOAD_ERROR"); //define LOAD_ABORTED?
    window._content.status = "Annotation loading stopped.";
};

Annozilla.prototype._addAnnotations = function (annotations, target) {
    
    this.asynchLoading = true; // TODO remove or make per-tab
    //var menuitem = window.document.getElementById('stop-load');
    //menuitem.hidden = 0;
    //this.operationInProgress("LOADING");

    // If "target" is specified, then use it and get the document from
    // that tab. Otherwise use the main window's document.
    // TODO should we always have "target" set?
    var doc = (target == undefined) ? window._content.document : target.contentDocument;
    
    var self = this;
    setTimeout(function () { self._addNextAnnotation (0, annotations, doc, target); }, 0);

};

Annozilla.prototype._addNextAnnotation = function (i, annotList, doc, target) {
    if (this.asynchLoading && (i < annotList.length)) {
        try {
            var annot = annotList[i]; //consider popping this?
            
            try {
                if (! annot.data) {
                    this._debugBadAnnotation(current);
                }
                else
                {
                    var context = annot.data.getProperty(Annozilla.ANNOT_NS,"context");
                    var shortURI = doc.documentURI.lastIndexOf("/")+1;
                    shortURI = doc.documentURI.substr(shortURI); 
                    if (context.indexOf(shortURI) == -1) {
                        //SKIP ANNOTATIONS STORED FOR OTHER PAGES
                        dump("(skip)");
                    } else {
                        var range = gAnnozillaXPointerUtils.xptrResolveAnnotation(annot, doc);
                        var model = new AnnozillaAnnotationModel (target);
    
                        // add to the overall collection (annotations for all servers)
                        model.addAnnotation (annot, range);
                    
                        if (range) {
                            this.icons.addAnnotation(annot, range, doc, target);
                            
                            // Only add the annotation to the sidebar if the annotation load
                            // is for the currently selected tab
                            if (this.sidebar != null && target == gBrowser.selectedBrowser) {
                                this.sidebar.addAnnotation(annot, range, doc);
                            }
                        }
                    }
                }
            } catch (e)
            {
                var consoleService=Components.classes['@mozilla.org/consoleservice;1'];
                var console = consoleService.getService(Components.interfaces.nsIConsoleService);
                console.logStringMessage("Annozilla: error loading annotations");
                Components.utils.reportError (e);
            }

            var self = this;
            setTimeout(function () { self._addNextAnnotation (i+1, annotList, doc, target); }, 0);
            
        } catch (e) {
            var consoleService=Components.classes['@mozilla.org/consoleservice;1'];
            var console = consoleService.getService(Components.interfaces.nsIConsoleService);
            console.logStringMessage("Annozilla: error loading annotations");
            Components.utils.reportError (e);
            //TODO: SIDEBAR
            //that.operationComplete("PAGE_LOAD_ERROR");
        }
    } else {
        //TODO: SIDEBAR
        //that.operationComplete("LOAD_OK");
        //var menuitem = window.document.getElementById('stop-load');
        //menuitem.hidden = 1;
        this.asynchLoading = false; // TODO make per-tab
        window._content.status = "Annotations loaded.";
        dump("\nDone!\n");
    }
};

// TODO can we unify _addAnnotations/_addReplies and
// _addNextAnnotation and _addNextReply?
Annozilla.prototype._addReplies = function (inReplyTo, repliesCount, replies, target) {
    
    this.asynchLoading = true; // TODO remove or make per-tab
    //var menuitem = window.document.getElementById('stop-load');
    //menuitem.hidden = 0;
    //this.operationInProgress("LOADING");

    var doc = target.contentDocument;

    if (repliesCount == 0)
    {
        if (this.sidebar != null)
        {
            this.sidebar.noRepliesFor (inReplyTo, target);
        }
    }
    else
    {
        var self = this;
        setTimeout(function () { self._addNextReply (replies, doc, target); }, 0);
    }

};

// For _addNextReply it is tricky to find the correct
// order to items. We don't just have an array of replies
// to a single annotation, we have a complete reply tree,
// so as we iterate through, we may have a reply to an item
// that we haven't added yet.
Annozilla.prototype._addNextReply =
function (replyList, doc, target) {
    if (this.asynchLoading && (replyList.length > 0)) {
        try {
            var annot = replyList.shift();
            if (! annot.data) {
                this._debugBadAnnotation(current);
            }
            else
            {
                var context = annot.data.getProperty(Annozilla.ANNOT_NS,"context");
                /* TODO what should replace this?
                var shortURI = doc.documentURI.lastIndexOf("/")+1;
                shortURI = doc.documentURI.substr(shortURI); 
                if (context.indexOf(shortURI) == -1) {
                    //SKIP ANNOTATIONS STORED FOR OTHER PAGES
                    dump("(skip)");
                } else {
                */
                    var range = gAnnozillaXPointerUtils.xptrResolveAnnotation(annot, doc);
                    
                    var threadRoot =
                        annot.data.getResourceProperty(
                            Annozilla.THREAD_NS, "root");
                    
                    // This isn't necessarily the same as the
                    // inReplyTo argument, since 'getReplies'
                    // loads the whole tree!
                    var inReplyToReal =
                        annot.data.getResourceProperty(
                            Annozilla.THREAD_NS, "inReplyTo");
 
                    var model = new AnnozillaAnnotationModel (target);
                    
                    if ( ! model.hasAnnotation (inReplyToReal))
                    {
                        Components.classes['@mozilla.org/consoleservice;1'].
                            getService(Components.interfaces.nsIConsoleService).
                            logStringMessage ("Can't find " + inReplyToReal + " so abandoning " + annot.annotationURI);
                        replyList.push (annot);
                        // TODO how to avoid infinite loops?
                    }

                    // add to the overall collection (annotations for all servers)
                    model.addAnnotation (annot, range, inReplyToReal, threadRoot);
      
                    if (range) {
                        
                        // We don't add an icon for a reply so don't call icon manager
                        
                        // Only add the annotation to the sidebar if the annotation load
                        // is for the currently selected tab
                        if (this.sidebar != null && target == gBrowser.selectedBrowser) {
                            this.sidebar.addReply (inReplyToReal, annot, range, doc, target);
                        }
                    }
                /*
                }
                */
            }

            var self = this;
            setTimeout(function () { self._addNextReply (replyList, doc, target); }, 0);
            
        } catch (e) {
            var consoleService=Components.classes['@mozilla.org/consoleservice;1'];
            var console = consoleService.getService(Components.interfaces.nsIConsoleService);
            console.logStringMessage("Annozilla: error loading annotations");
            Components.utils.reportError (e);
            //TODO: SIDEBAR
            //that.operationComplete("PAGE_LOAD_ERROR");
        }
    } else {

        //TODO: SIDEBAR
        //that.operationComplete("LOAD_OK");
        //var menuitem = window.document.getElementById('stop-load');
        //menuitem.hidden = 1;
        this.asynchLoading = false; // TODO make per-tab
        window._content.status = "Replies loaded.";
        dump("\nDone!\n");
    }
};
