/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Doug Daniels <rainking@alumni.rice.edu>
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Matthew Wilson <matthew@mjwilson.demon.co.uk>
 *  Eliot Setzer
 *  C. Greg Hagerty
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


function annozillaDeleteAnnotation(annotationURI){
    gAnnozilla.deleteOnCommand(annotationURI);
}

/**
 * Code to handle context menu's "Annotate selection" option.
 */

/* *******************************************************************
 * Quick NonModal Post Annotation - doesn't create edit dialog
 *   adapted from editor/common.js: PostObserver..post
 *
 */
function nonModalPostAnnotation(baseURL, xptr, type, content) {
    if (!type) {type = "Comment";}
    if (!content) {content = "";}
    var Annozilla;
    try {Annozilla = gAnnozilla;} catch (e) {}
    if (!Annozilla) {
        try {
            //argh, called from sidebar
            var mainWindow = window
                .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                .getInterface(Components.interfaces.nsIWebNavigation)
                .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                .rootTreeItem
                .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                .getInterface(Components.interfaces.nsIDOMWindow);
            Annozilla = mainWindow.gAnnozilla;
        } catch (e) {}
    }
    if (!Annozilla) {
        window.alert("Annozilla not loaded?\n");
        return;
    }

    /*
    var service = Components.classes["@mozilla.org/annotations_service;1"]
        .getService()
        .QueryInterface(Components.interfaces.nsIAnnotationsService);
    var app = service.registerApplication("annozilla");
    */
    var app = Annozilla.app; //annotations_service
    var server = app.getDefaultPostServer();
    if (!server) { 
        //see commonEditor.js: new scheme blows away default
        // have to go to PostObserver.SERVERS_FILENAME to figure it out
        server = "urn:localserver"; 
        dump("Post server defaulting to localserver\n");
    }
    if (! app.hasServer(server)) { //gAnnozilla.app
        //window.alert(this.editorBundle.formatStringFromName
        //("SERVER_ERROR3", , 1));
        //("SERVER_ERROR1", [this.annotation.serverURI], 1));
        window.alert("There is no current annotation server '"+server+"'?\n");
        return;
    }
    var date = formatDate();
    var charset = window.QueryInterface(Components.interfaces.nsIDOMWindow)
        .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
        .getInterface(Components.interfaces.nsIDocCharset)
        .charset;
    var data = Components.classes["@mozilla.org/annotation-metadata;1"]
        .createInstance(Components.interfaces.nsIAnnotationMetadata);

    var creator = Annozilla.prefs.getPref("creator");
    if (!creator) { 
        creator = "Anonymous";
        dump("Default creator: Anonymous?\n");
    }
    data.setRequiredProperties
        ("http://www.w3.org/2000/10/annotation-ns#Annotation",
         "http://www.w3.org/2000/10/annotationType#" + type, 
         baseURL, 
         baseURL + "#" + xptr,
         creator,
         date,
         date);
    data.setLiteralProperty("http://purl.org/dc/elements/1.0/",
                            "language",
                            Annozilla.prefs.getPref("language"));
    data.setResourceProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#",
                             "type", 
                             "http://www.w3.org/2000/10/annotationType#"
                             + type);
    
    // test content
    var xml = '<html xmlns="http://www.w3.org/1999/xhtml"><body>'
        +content+"</body></html>";
    
    app.postAnnotation(data, xml, charset, server);
} 


//copied from date.js
//  though editor/common.js/PostObserver.formatDate() is more fancy looking
function formatDate() {

  var now  = new Date();

  // who'd have thought you could end up missing printf?
  return now.getUTCFullYear() + "-" +
         new String(now.getUTCMonth() + 1).replace(/^([0-9])$/, "0$1") + "-" +
         new String(now.getUTCDate()).replace(/^([0-9])$/, "0$1") + " at " +
         new String (now.getUTCHours()).replace(/^([0-9])$/, "0$1") + ":" +
         new String(now.getUTCMinutes()).replace(/^([0-9])$/, "0$1") + ":" +
         new String(now.getUTCSeconds()).replace(/^([0-9])$/, "0$1");
}


/* ***************************************************************************
 * Code to handle context menu's "Annotate selection" option.
 */

function annozillaAnnotateSelection(isFirefox) {
  annozillaAnnotateSelectionAs(null, isFirefox, false);
}

/*
 * allow type of annotation to be specified 
 *  and passed instead of a null annotation arg to dialog
 */
function annozillaAnnotateSelectionAs(type, isFirefox, nonModalP) {
    if (! Components.classes["@mozilla.org/xpointer-service;1"]) {
	window.alert("Please install the XPointerService from http://xpointerlib.mozdev.org/");
        return;
    }
    
    //var focusedWindow = document.commandDispatcher.focusedWindow;
    var focusedWindow = window._content;
    var wrapper = new XPCNativeWrapper(focusedWindow, 'getSelection()');
    var url = annozillaStripURL(focusedWindow.location.href);

    var seln = wrapper.getSelection();
    var doc = focusedWindow.document;

    /* it's okay now, added annotation becomes "global" page annotation
    if (seln.anchorNode == null) {
        dump("No selection in "+url+" - not adding annotation\n");
    } else {
        dump("anchorNode: "+seln.anchorNode.nodeName
             + " anchorOffset: "+seln.anchorOffset
             + " parent:"+seln.anchorNode.parentNode.nodeName
             + " focusNode: "+seln.focusNode.nodeName
             + " focusOffset: "+seln.focusOffset
             + " in "+url
             + "\n"
             );
    }
    */

    var xptrService = Components.classes["@mozilla.org/xpointer-service;1"].
           getService().QueryInterface(Components.interfaces.nsIXPointerService);
    var xptr = xptrService.createXPointerFromSelection(wrapper.getSelection(),
                                                       focusedWindow.document);

    if (nonModalP) {
        nonModalPostAnnotation(url, xptr, type);
    }
    else
    {
        window.top.gAnnozilla.openEditorWindow (
            "chrome,resizable,dialog=no",
                type,
                url,
                xptr,
                null, // bodyURI
                null, // inReplyTo,
                null, // inReplyToBodyURI,
                null, // threadRoot,
                gBrowser.selectedBrowser);
    }
}

/**
 * Private function to strip query strings off of URLs. (we don't want to be
 * nosy!)
 * Also strips off fragment ids
 * TODO Duplicated in annozilla.js.
 *
 * @param url String URL to strip query strings from
 * @return URL string stripped of queries and fragment ids
 */
function annozillaStripURL(url) {
  if (!url || ('string' != typeof(url)) ||  ('' == url) || url == 'about:blank') {
    return "about:blank";
  }
  
  // Strip off any query strings (Don't want to be too nosy).
  var queryPart = url.indexOf('?');
  if (queryPart != -1) {
    url = url.slice(0, queryPart);
  }

  url = url.replace (/#.*/,'');

  return url;    
}


/* *******************************************************************
 * Edit selected annotation - there might be a choice
 */

// note: May want to toggle Firefox "caret browsing" with F7 
// or Options->Advanced->General Always use cursor keys to navagate...
function editSelectedAnnotation() {
    var focusedWindow = document.commandDispatcher.focusedWindow;
    var wrapper = new XPCNativeWrapper(focusedWindow, 'getSelection()');
    var url = annozillaStripURL(focusedWindow.location.href);

    var seln = wrapper.getSelection();
    var doc = focusedWindow.document;
    if (seln.anchorNode == null) {
        window._content.status = "No selection.\n";
        //dump("No selection in "+url+"\n");
        return;
    } else {
        dump("anchorNode: "+seln.anchorNode.nodeName
             + " anchorOffset: "+seln.anchorOffset
             + " parent:"+seln.anchorNode.parentNode.nodeName
             + " focusNode: "+seln.focusNode.nodeName
             + " focusOffset: "+seln.focusOffset
             + " in "+url
             + "\n"
             );
    }

    var node = seln.anchorNode;
    var uriList = [];
    while (node) {
        if ((node.nodeType == node.ELEMENT_NODE)
            && node.hasAttribute("annotation-id")) {
            uriList.push(node.getAttribute("annotation-id"));
        }
        node = node.parentNode;
    }

    if (uriList.length > 0) {
        if (uriList.length > 1) {
            window._content.status = "Innermost of "+uriList.length
                +" annotations, "+uriList[0];
        } else {
            window._content.status = "Annotation "+uriList[0];
        }
        gAnnozilla.icons.iconClick(gAnnozilla.annotations[uriList[0]].annot);
    } else {
        window._content.status = "No annotations selected.";
    }
}
