/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function getEditorWrapper () {
   return new MidasEditorWrapper();
}

function MidasEditorWrapper () {
}

MidasEditorWrapper.prototype.ENCODE_ORDER = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

MidasEditorWrapper.prototype.base64ToAscii = function (c)
{
    return this.ENCODE_ORDER.substring(c,c+1);
}

MidasEditorWrapper.prototype.encodeBase64 = function (str) {
	var result = "";
	var i = 0;
	var sextet = 0;
	var leftovers = 0;
	var octet = 0;

	for (i=0; i < str.length; i++) {
		octet = str.charCodeAt(i);
		switch( i % 3 )
		{
			case 0:
			{
				sextet = ( octet & 0xFC ) >> 2 ;
				leftovers = octet & 0x03 ;

				// sextet contains first character in quadruple
				break;
			}

			case 1:
			{
				sextet = ( leftovers << 4 ) | ( ( octet & 0xF0 ) >> 4 );
				leftovers = octet & 0x0F ;

				// sextet contains 2nd character in quadruple
				break;
			}

			case 2:
			{
				sextet = ( leftovers << 2 ) | ( ( octet & 0xC0 ) >> 6 ) ;
				leftovers = ( octet & 0x3F ) ;

				// sextet contains third character in quadruple
				// leftovers contains fourth character in quadruple
				break;
			}
		}

		result = result + this.base64ToAscii(sextet);

		// don't forget about the fourth character if it is there
		if( (i % 3) == 2 )
		{
			result = result + this.base64ToAscii(leftovers);
		} 

	}

	// figure out what to do with leftovers and padding
	switch( str.length % 3 )
	{
		case 0:
		{
			// an even multiple of 3, nothing left to do
			break ;
		}
		case 1:
		{
			// one 6-bit chars plus 2 leftover bits
			leftovers =  leftovers << 4 ;
			result = result + this.base64ToAscii(leftovers);
			result = result + "==";
			break ;
		}
		case 2:
		{
			// two 6-bit chars plus 4 leftover bits
			leftovers = leftovers << 2 ;
			result = result + this.base64ToAscii(leftovers);
			result = result + "=";
			break ;
		}
	}

	return result;
}


MidasEditorWrapper.prototype.createTempURIFromString = function (str, type) {   

    var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
        .createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    converter.charset = "UTF-8";

    var uri = "data:" + type + ";base64," + this.encodeBase64 (converter.ConvertFromUnicode(str));

    return uri;
}


MidasEditorWrapper.prototype.loadUrl = function (url, afterload) {
    /* In Firefox 1.5, we can't load annotations with content type
       application/xhtml+xml. So load the annotation body using XMLHttpRequest
       and write the result into the editing frame. */
    var bodyLoader = new XMLHttpRequest();
    bodyLoader.console = Components.classes['@mozilla.org/consoleservice;1']
        .getService(Components.interfaces.nsIConsoleService);
    bodyLoader.open ("GET", url);
    var that = this;
    bodyLoader.onload = function () {
        // Can't use document.write for security reasons.
        var url = that.createTempURIFromString(bodyLoader.responseText, "text/html");
        document.getElementById ("edit-frame").webNavigation.loadURI (url, 65535, null, null, null);
        // Seem to need a timeout before switching design mode on.
        setTimeout (function () {designModeOn(); afterload()}, 100);
    };
    bodyLoader.onerror = function (e) { 
        var editorBundle = srGetStrBundle("chrome://annozilla/locale/editor.properties");
        window.alert (editorBundle.GetStringFromName ("DOWNLOAD_ERROR"));
        this.console.logStringMessage ("Download error for " + url + "; status="+e.target.status + " status text="+e.target.statusText);
    };
    bodyLoader.send ('');
}

MidasEditorWrapper.prototype.getTitle = function () {
   return document.getElementById("edit-frame").contentDocument.title;
}

MidasEditorWrapper.prototype.setTitle = function (title) {
   return document.getElementById("edit-frame").contentDocument.title = title;
}

MidasEditorWrapper.prototype.getDocument = function () {
    var serializer = new XMLSerializer();
    var serialized = serializer.serializeToString (document.getElementById("edit-frame").contentWindow.document);
    // remove doctype
    serialized = serialized.replace (/<!DOCTYPE[^>]*>/,'');

    // This is hacky. Why do we need to lower-casing of XHTML elements?
    // (This doesn't even do attributes...)
    serialized = serialized.replace
        ( /<(\/?\w+)/g,
          function (s1) {
          return s1.toLowerCase();
          }
        );

    // remove _moz attributes
    serialized = serialized.replace (/\s_moz_[a-z_]*="[^"]*"/g,'');

    // remove type="_moz" attribute
    serialized = serialized.replace (/type="_moz"/g,'');

    // Ensure namespace on HTML element
    serialized = serialized.replace (/<html[^>]*?>/, '<html xmlns="http://www.w3.org/1999/xhtml">');
  
    return serialized;
}
