/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function AnnozillaAnnotationModel (browser)
{
    this.browser = browser;
}

AnnozillaAnnotationModel.ANNOZILLA_ANNOTATIONS_KEY = "annozilla-annotations";
AnnozillaAnnotationModel.ANNOZILLA_SERVERS_REMAINING_KEY = "annozilla-serversremaining";

AnnozillaAnnotationModel.prototype.getAnnotations =
function ()
{
    var model = this.browser.getUserData (AnnozillaAnnotationModel.ANNOZILLA_ANNOTATIONS_KEY);
    if (model == null)
    {
        return new Object();
    }
    else
    {
        return model;
    }
};

AnnozillaAnnotationModel.prototype.setAnnotations =
function (annotations)
{
    this.browser.setUserData (
        AnnozillaAnnotationModel.ANNOZILLA_ANNOTATIONS_KEY,
        annotations,
        function (operation, key, data, source, destination) { } );
};

AnnozillaAnnotationModel.prototype.clearAnnotations = function ()
{
    this.setAnnotations (new Object());
};

/**
 * Add an annotation to the data model.
 * "inReplyTo" and "threadRoot" should only be specified for replies
 * and can be omitted otherwise.
 */
AnnozillaAnnotationModel.prototype.addAnnotation =
function (aAnnotation, aRange, inReplyTo, threadRoot)
{

    var currentAnnotations = this.getAnnotations();
    
    currentAnnotations[aAnnotation.annotationURI] = {
        annot: aAnnotation,
        range: aRange,
        inReplyTo: inReplyTo,
        threadRoot: threadRoot        
    };

    this.setAnnotations (currentAnnotations);
};

AnnozillaAnnotationModel.prototype.updateAnnotationWithRange =
function (aAnnotation, aRange)
{

    var currentAnnotations = this.getAnnotations();
    currentAnnotations[aAnnotation.annotationURI].range = aRange;
    this.setAnnotations (currentAnnotations);
};

AnnozillaAnnotationModel.prototype.deleteAnnotation =
function (url)
{
    
    var currentAnnotations = this.getAnnotations();   
    if (currentAnnotations[url]) {
        delete currentAnnotations[url];
    }
    this.setAnnotations (currentAnnotations);

};

AnnozillaAnnotationModel.prototype.updateAnnotation =
function (updatedAnnotation)
{
    var currentAnnotations = this.getAnnotations();
    var uri = updatedAnnotation.annotationURI;
    if (currentAnnotations && currentAnnotations[uri]) {
        currentAnnotations[uri].annot = updatedAnnotation;
    }
    this.setAnnotations (currentAnnotations);

};

AnnozillaAnnotationModel.prototype.hasAnnotation =
function (uri)
{
    var currentAnnotations = this.getAnnotations();
    return (uri != undefined &&
            currentAnnotations != undefined &&
            currentAnnotations[uri] != undefined);

};

AnnozillaAnnotationModel.prototype.getAnnotation =
function (uri)
{
    var currentAnnotations = this.getAnnotations();
    return currentAnnotations[uri];
};

AnnozillaAnnotationModel.prototype.startOperation =
function (serverTotal)
{
    var currentCount =
        this.browser.getUserData (
           AnnozillaAnnotationModel.ANNOZILLA_SERVERS_REMAINING_KEY);
           
    if (currentCount == undefined) { currentCount = 0; }
    
    // TODO Atomicity?

    currentCount += serverTotal;

    this.browser.setUserData (
        AnnozillaAnnotationModel.ANNOZILLA_SERVERS_REMAINING_KEY,
        currentCount,
        function (operation, key, data, source, destination) { } );

    //this.browser.setUserData
    //   (AnnozillaAnnotationModel.ANNOZILLA_SERVER_TOTAL_KEY,
    //    serverTotal,
    //    function (operation, key, data, source, destination) { } );
    
    return currentCount;
};

AnnozillaAnnotationModel.prototype.serverResponded =
function ()
{
    var serverCount =
        this.browser.getUserData (AnnozillaAnnotationModel.ANNOZILLA_SERVERS_REMAINING_KEY);
    //var serverTotal =
      //  this.browser.getUserData (AnnozillaAnnotationModel.ANNOZILLA_SERVER_TOTAL_KEY);
    serverCount--;

    this.browser.setUserData (
        AnnozillaAnnotationModel.ANNOZILLA_SERVERS_REMAINING_KEY,
        serverCount,
        function (operation, key, data, source, destination) { } );
    return serverCount;
};